@@ -373,7 +373,7 @@ MiscInitialization (
       DEBUG ((
         DEBUG_ERROR,
         "%a: Unknown Host Bridge Device ID: 0x%04x\n",
-        __FUNCTION__,
+        __func__,
         mHostBridgeDevId
         ));
       ASSERT (FALSE);
@@ -501,17 +501,17 @@ S3Verification (
     DEBUG ((
       DEBUG_ERROR,
       "%a: S3Resume2Pei doesn't support X64 PEI + SMM yet.\n",
-      __FUNCTION__
+      __func__
       ));
     DEBUG ((
       DEBUG_ERROR,
       "%a: Please disable S3 on the QEMU command line (see the README),\n",
-      __FUNCTION__
+      __func__
       ));
     DEBUG ((
       DEBUG_ERROR,
       "%a: or build OVMF with \"OvmfPkgIa32X64.dsc\".\n",
-      __FUNCTION__
+      __func__
       ));
     ASSERT (FALSE);
     CpuDeadLoop ();
@@ -558,7 +558,7 @@ MaxCpuCountInitialization (
   DEBUG ((
     DEBUG_INFO,
     "%a: QEMU reports %d processor(s)\n",
-    __FUNCTION__,
+    __func__,
     ProcessorCount
     ));
 }
