@@ -20,7 +20,6 @@
 
 #include "SnpPageStateChange.h"
 
-#define IS_ALIGNED(x, y)  ((((x) & (y - 1)) == 0))
 #define PAGES_PER_LARGE_ENTRY  512
 
 STATIC
@@ -114,7 +113,7 @@ PvalidateRange (
         DEBUG_ERROR,
         "%a:%a: Failed to %a address 0x%Lx Error code %d\n",
         gEfiCallerBaseName,
-        __FUNCTION__,
+        __func__,
         Validate ? "Validate" : "Invalidate",
         Address,
         Ret
@@ -238,7 +237,7 @@ InternalSetPageState (
     DEBUG_VERBOSE,
     "%a:%a Address 0x%Lx - 0x%Lx State = %a LargeEntry = %d\n",
     gEfiCallerBaseName,
-    __FUNCTION__,
+    __func__,
     BaseAddress,
     EndAddress,
     State == SevSnpPageShared ? "Shared" : "Private",
