@@ -269,13 +269,6 @@ PcRtcInit (
   Time.Month  = RtcRead (RTC_ADDRESS_MONTH);
   Time.Year   = RtcRead (RTC_ADDRESS_YEAR);
 
-  //
-  // Set RTC configuration after get original time
-  // The value of bit AIE should be reserved.
-  //
-  RegisterB.Data = FixedPcdGet8 (PcdInitialValueRtcRegisterB) | (RegisterB.Data & BIT5);
-  RtcWrite (RTC_ADDRESS_REGISTER_B, RegisterB.Data);
-
   //
   // Release RTC Lock.
   //
@@ -324,12 +317,20 @@ PcRtcInit (
     Time.Hour       = RTC_INIT_HOUR;
     Time.Day        = RTC_INIT_DAY;
     Time.Month      = RTC_INIT_MONTH;
-    Time.Year       = PcdGet16 (PcdMinimalValidYear);
+    Time.Year       = MAX (PcdGet16 (PcdRtcDefaultYear), PcdGet16 (PcdMinimalValidYear));
+    Time.Year       = MIN (Time.Year, PcdGet16 (PcdMaximalValidYear));
     Time.Nanosecond = 0;
     Time.TimeZone   = EFI_UNSPECIFIED_TIMEZONE;
     Time.Daylight   = 0;
   }
 
+  //
+  // Set RTC configuration after get original time
+  // The value of bit AIE should be reserved.
+  //
+  RegisterB.Data = FixedPcdGet8 (PcdInitialValueRtcRegisterB) | (RegisterB.Data & BIT5);
+  RtcWrite (RTC_ADDRESS_REGISTER_B, RegisterB.Data);
+
   //
   // Reset time value according to new RTC configuration
   //
@@ -344,7 +345,7 @@ PcRtcInit (
   // so we can use them to get and set wakeup time.
   //
   Status = PcRtcGetWakeupTime (&Enabled, &Pending, &Time, Global);
-  if ((Enabled) || (!EFI_ERROR (Status))) {
+  if ((!EFI_ERROR (Status)) || (Enabled)) {
     return EFI_SUCCESS;
   }
 
@@ -357,7 +358,8 @@ PcRtcInit (
   Time.Hour       = RTC_INIT_HOUR;
   Time.Day        = RTC_INIT_DAY;
   Time.Month      = RTC_INIT_MONTH;
-  Time.Year       = PcdGet16 (PcdMinimalValidYear);
+  Time.Year       = MAX (PcdGet16 (PcdRtcDefaultYear), PcdGet16 (PcdMinimalValidYear));
+  Time.Year       = MIN (Time.Year, PcdGet16 (PcdMaximalValidYear));
   Time.Nanosecond = 0;
   Time.TimeZone   = Global->SavedTimeZone;
   Time.Daylight   = Global->Daylight;
@@ -836,8 +838,11 @@ PcRtcSetWakeupTime (
     //
     // Just support set alarm time within 24 hours
     //
-    PcRtcGetTime (&RtcTime, &Capabilities, Global);
-    Status = RtcTimeFieldsValid (&RtcTime);
+    Status = PcRtcGetTime (&RtcTime, &Capabilities, Global);
+    if (!EFI_ERROR (Status)) {
+      Status = RtcTimeFieldsValid (&RtcTime);
+    }
+
     if (EFI_ERROR (Status)) {
       return EFI_DEVICE_ERROR;
     }
@@ -995,13 +1000,16 @@ ConvertRtcTimeToEfiTime (
   BOOLEAN  IsPM;
   UINT8    Century;
 
-  if ((Time->Hour & 0x80) != 0) {
-    IsPM = TRUE;
-  } else {
-    IsPM = FALSE;
-  }
+  // IsPM only makes sense for 12-hour format.
+  if (RegisterB.Bits.Mil == 0) {
+    if ((Time->Hour & 0x80) != 0) {
+      IsPM = TRUE;
+    } else {
+      IsPM = FALSE;
+    }
 
-  Time->Hour = (UINT8)(Time->Hour & 0x7f);
+    Time->Hour = (UINT8)(Time->Hour & 0x7f);
+  }
 
   if (RegisterB.Bits.Dm == 0) {
     Time->Year   = CheckAndConvertBcd8ToDecimal8 ((UINT8)Time->Year);
